/* 
   Affix - Bluetooth Protocol Stack for Linux
   Copyright (C) 2001,2002 Nokia Corporation
   Author: Dmitry Kasatkin <dmitry.kasatkin@nokia.com>

   Original Author: Guruprasad Krishnamurthy <kgprasad@hotmail.com>

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   This program is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License along
   with this program; if not, write to the Free Software Foundation, Inc.,
   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*
   $Id: attr.c,v 1.16 2002/11/05 16:32:27 kds Exp $

   Fixes:
   		Dmitry Kasatkin		: cleanup, fixes
*/

#include <affix/config.h>

#include <stdio.h>
#include <malloc.h>
#include <string.h>
#include <stdint.h>
#include <netinet/in.h>

#include <affix/sdp.h>
#include <affix/sdpclt.h>
#include "attr.h"
#include "utils.h"

void removeUUIDFromTargetPattern(ServiceRecord *svcRec, slist_t *pSeq)
{
}

void addUUIDToTargetPattern(ServiceRecord *svcRec, UUID *pUUID)
{
	UUID *pUUID128Bit;

//	if (sdp_mode == SDP_CLIENT)
//		return;
	
	if ((svcRec == NULL) || (pUUID == NULL))
		return;

	pUUID128Bit = convertTo128Bits(pUUID);
	BTDEBUG("SvcRec : 0x%lx", (unsigned long)svcRec);
	BTDEBUG("Elements in target pattern : %d", s_list_length(svcRec->targetPattern));
	BTDEBUG("Trying to add : 0x%lx", (unsigned long)pUUID128Bit);
	if (s_list_find_custom(svcRec->targetPattern, pUUID128Bit, compareUUID) == NULL)
		s_list_insert_sorted(&svcRec->targetPattern, pUUID128Bit, compareUUID);
	else
		free(pUUID128Bit);
	BTDEBUG("Elements in target pattern : %d", s_list_length(svcRec->targetPattern));
}

/*
 ** Sets the length of a data element sequence
 ** The size is either an 8 bit quantity or a 16 bit
 ** quantity. This method is called after the actual 
 ** data is appended to the data element sequence and
 ** PDU length is computed
 */
void setSequenceLengthPDU(PDUForm *pduForm)
{
	char			*pdata;
	DataTypeDescriptor	dataType;

	pdata = pduForm->data;
	dataType = __get_u8(pdata);
	pdata += sizeof(DataTypeDescriptor);

	switch (dataType) {
		case SequenceSize8 :
			__put_u8(pdata, pduForm->dataSize - 
				sizeof(DataTypeDescriptor) - sizeof(DataSizeDescriptor8));
			break;

		case SequenceSize16 :
			__put_u16(pdata, htons(pduForm->dataSize - 
					sizeof(DataTypeDescriptor) - sizeof(DataSizeDescriptor16)));
			break;

		case SequenceSize32 :
			__put_u32(pdata, htonl(pduForm->dataSize - 
					sizeof(DataTypeDescriptor) - sizeof(DataSizeDescriptor32)));
			break;
		default :
			break;
	}
}


/*
 ** This function appends data to the PDU form in the buffer "dst"
 ** from source "src". The data length is also computed and set.
 ** Should the PDU form length exceed 2^8, then sequence type is
 ** set accordingly and the data is memmove()'d.
 */
int appendToPduForm(PDUForm *dst, PDUForm *src)
{
	DataTypeDescriptor dataType;
	char *pdata;

	pdata = dst->data;
	dataType = __get_u8(pdata);

	BTDEBUG("Append src size : %d", src->dataSize);
	BTDEBUG("Append dst size : %d", dst->dataSize);
	BTDEBUG("Dst buffer size : %d", dst->bufferSize);

	if ((dst != NULL) && (dst->data != NULL)) {
		if ((dst->dataSize + src->dataSize) > dst->bufferSize) {
			/*
			 ** Need to realloc !
			 */
			int neededSize = SDP_PDU_CHUNK_SIZE * ((src->dataSize/SDP_PDU_CHUNK_SIZE) + 1);
			dst->data = (char *)realloc(dst->data, (dst->bufferSize + neededSize));
			BTDEBUG("Realloc'ing : %d", neededSize);
			if (dst->data == NULL) {
				BTERROR("Realloc fails \n");
			}
			dst->bufferSize += neededSize;
		}
		if ((dst->dataSize == 0) && (dataType == 0)) {
			//create initial sequence
			__put_u8(pdata, SequenceSize8);
			pdata += sizeof(DataTypeDescriptor);
			dst->dataSize += sizeof(DataTypeDescriptor);
			/*
			 ** Space for sequence size
			 */
			pdata += sizeof(DataSizeDescriptor8);
			dst->dataSize += sizeof(DataSizeDescriptor8);
		}

		memcpy((dst->data + dst->dataSize), src->data, src->dataSize);
		dst->dataSize += src->dataSize;

		dataType = *(DataTypeDescriptor *)dst->data;
		if ((dst->dataSize > UCHAR_MAX) && (dataType == SequenceSize8)) {
			/*
			 ** Need to memmove
			 */
			short offset = sizeof(DataTypeDescriptor) + sizeof(DataSizeDescriptor8);
			memmove((dst->data + offset + 1), (dst->data + offset),
					(dst->dataSize - offset));
			pdata = dst->data;
			__put_u8(pdata, SequenceSize16);
			pdata += sizeof(DataTypeDescriptor);
			dst->dataSize += 1;
		}
	}

	setSequenceLengthPDU(dst);

	return src->dataSize;
}

void printPduForm(PDUForm *pduForm)
{
	if (pduForm->dataSize == 0)
		BTDEBUG("NULL buffer");
	BTDUMP(pduForm->data, pduForm->dataSize);
}

int isSequenceData(SDPData *pSDPData)
{
	if ((pSDPData->dtd >> 3) == 6)
		return 1;
	return 0;
}

int isAlternativeData(SDPData *pSDPData)
{
	if ((pSDPData->dtd >> 3) == 7)
		return 1;
	return 0;
}

int verifySDPData(SDPData *pSDPData)
{
	return 1;
}

unsigned int attrIdHashFunc(const void * key)
{
	unsigned int attrId;

	attrId = *(ServiceAttributeIdentifier *)key;
	return attrId;
}

void freeSDPData(SDPData *pSDPData)
{
	if ((pSDPData->dtd & 0x0F) == 0x30) {
		freeSDPDataSeq(pSDPData->sdpDataValue.dataSeq);
	} else if ((pSDPData->dtd & 0x0F) == 0x20 || (pSDPData->dtd & 0x0F) == 0x40) {
		/* string type */
		free(pSDPData->sdpDataValue.stringPtr);
	}
	free(pSDPData);
}

void freeSDPDataSeq(slist_t *pDataSeq)
{
	slist_t	*dataSeq;
	for (dataSeq = pDataSeq; dataSeq; dataSeq = s_list_next(dataSeq)) {
		freeSDPData(dataSeq->data);
	}
	s_list_free(&pDataSeq);
}

SDPData *createSDPData(DataTypeDescriptor dtd, void *pValue)
{
	SDPData		*pSeqEntry;
	SDPData		*pSDPData = NULL;
	int		strLength = 0;
	slist_t		*dataSeq;

	pSDPData = (SDPData *)malloc(sizeof(SDPData));
	if (pSDPData == NULL)
		return NULL;
	memset(pSDPData, 0, sizeof(SDPData));

	pSDPData->dtd = dtd;
	pSDPData->unitSize = sizeof(DataTypeDescriptor);

	switch (dtd) {
		case UnsignedInteger_8 :
			pSDPData->sdpDataValue.uint8 = *(uint8_t *)pValue;
			pSDPData->unitSize += sizeof(uint8_t);
			break;
		case Integer_8 :
		case Boolean :
			pSDPData->sdpDataValue.int8 = *(int8_t *)pValue;
			pSDPData->unitSize += sizeof(int8_t);
			break;
		case UnsignedInteger_16 :
			pSDPData->sdpDataValue.uint16 = *(uint16_t *)pValue;
			pSDPData->unitSize += sizeof(uint16_t);
			break;
		case Integer_16 :
			pSDPData->sdpDataValue.int16 = *(int16_t *)pValue;
			pSDPData->unitSize += sizeof(int16_t);
			break;
		case UnsignedInteger_32 :
			pSDPData->sdpDataValue.uint32 = *(uint32_t *)pValue;
			pSDPData->unitSize += sizeof(uint32_t);
			break;
		case Integer_32 :
			pSDPData->sdpDataValue.int32 = *(int32_t *)pValue;
			pSDPData->unitSize += sizeof(int32_t);
			break;
		case Integer_64 :
			pSDPData->sdpDataValue.int64 = *(int64_t *)pValue;
			pSDPData->unitSize += sizeof(int64_t);
			break;
		case UnsignedInteger_64 :
			pSDPData->sdpDataValue.uint64 = *(uint64_t *)pValue;
			pSDPData->unitSize += sizeof(uint64_t);
			break;
		case UnsignedInteger_128 :
			memcpy(&pSDPData->sdpDataValue.uint128, pValue, sizeof(Int128Bit));
			pSDPData->unitSize += sizeof(int64_t);
			break;
		case Integer_128 :        
			memcpy(&pSDPData->sdpDataValue.int128, pValue, sizeof(Int128Bit));
			pSDPData->unitSize += sizeof(uint64_t);
			break;
		case UUID_16 :
			makeUUIDFrom16Bits(&pSDPData->sdpDataValue.uuid, *(UUID16Bit *)pValue);
			pSDPData->unitSize += sizeof(UUID16Bit);
			break;
		case UUID_32 :
			makeUUIDFrom32Bits(&pSDPData->sdpDataValue.uuid, *(UUID32Bit *)pValue);
			pSDPData->unitSize += sizeof(UUID32Bit);
			break;
		case UUID_128 :
			makeUUIDFrom128Bits(&pSDPData->sdpDataValue.uuid, (UUID128Bit *)pValue);
			pSDPData->unitSize += sizeof(UUID128Bit);
			break;
		case URLStringSize8 :
		case TextStringSize8 :
		case URLStringSize16 :
		case TextStringSize16 :
			strLength = strlen(pValue) + 1;
			if (strLength > USHRT_MAX) {
				BTERROR("Strings of size > USHRT_MAX not supported\n");
				goto err;
			}
			pSDPData->unitSize += strLength;
			pSDPData->sdpDataValue.stringPtr = (char*)malloc(strLength);
			if (!pSDPData->sdpDataValue.stringPtr)
				goto err;
			strcpy(pSDPData->sdpDataValue.stringPtr, pValue);
			if (strLength <= UCHAR_MAX) {
				pSDPData->unitSize += sizeof(uint8_t);
				if ((dtd != URLStringSize8) && (dtd != TextStringSize8)) {
					if (dtd == URLStringSize16)
						dtd = URLStringSize8;
					else
						dtd = TextStringSize8;
				}
			} else {
				pSDPData->unitSize += sizeof(uint16_t);
				if ((dtd != URLStringSize16) && (dtd != TextStringSize16)) {
					if (dtd == URLStringSize8)
						dtd = URLStringSize16;
					else
						dtd = TextStringSize16;
				}
			}
			break;
		case URLStringSize32 :
		case TextStringSize32 :
			BTERROR("Strings of size > USHRT_MAX not supported\n");
			goto err;
			break;
		case AlternativeSize8 :
		case AlternativeSize16 :
		case AlternativeSize32 :
		case SequenceSize8 :
		case SequenceSize16 :
		case SequenceSize32 :
			pSDPData->sdpDataValue.dataSeq = pValue;
			for (dataSeq = pValue; dataSeq; dataSeq = s_list_next(dataSeq)) {
				pSeqEntry = dataSeq->data;
				pSDPData->unitSize += pSeqEntry->unitSize;
			}
			break;
		default :
			goto err;
			break;
	}
	return pSDPData;
err:
	free(pSDPData);
	return NULL;	
}

void addSDPDataSeq(SDPData *pSeq, SDPData *pSDPData)
{

	s_list_append(&pSeq->sdpDataValue.dataSeq, pSDPData);
	pSeq->unitSize += pSDPData->unitSize;
}


void removeSDPAttribute(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId)
{
	SDPData *pSDPData = NULL;

	if ((svcRec == NULL) || (svcRec->attributeList == NULL))
		return;

	pSDPData = getSDPData(svcRec, attrId);
	if (pSDPData != NULL)
		s_list_remove(&svcRec->attributeList, pSDPData);
}


void setSequenceLength(char *sequenceStartPtr, int length)
{
	DataTypeDescriptor dtd;

	dtd = __get_u8(sequenceStartPtr);
	switch (dtd) {
		case SequenceSize8 :
		case AlternativeSize8 :
		case TextStringSize8 :
		case URLStringSize8 :
			__put_u8(sequenceStartPtr + 1, length);
			break;

		case SequenceSize16 :
		case AlternativeSize16 :
		case TextStringSize16 :
		case URLStringSize16 :
			__put_u16(sequenceStartPtr + 1, htons(length));
			break;

		case SequenceSize32 :
		case AlternativeSize32 :
		case TextStringSize32 :
		case URLStringSize32 :
			__put_u32(sequenceStartPtr + 1, htonl(length));
			break;
	}
}

int setDataTypeDescriptor(PDUForm *pduForm, DataTypeDescriptor dtd)
{
	char *pdata;
	int dataSize = 0;
	int originalSize = 0;

	if (pduForm == NULL)
		return 0;

	originalSize = pduForm->dataSize;
	pdata = pduForm->data + pduForm->dataSize;
	__put_u8(pdata, dtd);
	pdata += sizeof(DataTypeDescriptor);
	pduForm->dataSize += sizeof(DataTypeDescriptor);

	switch (dtd) {
		case SequenceSize8 :
		case TextStringSize8 :
		case URLStringSize8 :
		case AlternativeSize8 :
			pduForm->dataSize += sizeof(uint8_t);
			break;
		case SequenceSize16 :
		case TextStringSize16 :
		case URLStringSize16 :
		case AlternativeSize16 :
			pduForm->dataSize += sizeof(uint16_t);
			break;
		case SequenceSize32 :
		case TextStringSize32 :
		case URLStringSize32 :
		case AlternativeSize32 :
			pduForm->dataSize += sizeof(uint32_t);
			break;
	}

	dataSize = pduForm->dataSize - originalSize;

	return dataSize;
}


void setAttributeId(PDUForm *pduForm, ServiceAttributeIdentifier svcAttrId)
{
	char *pdata;

	if (pduForm == NULL)
		return;

	pdata = pduForm->data;

	/* Data type for SvcAttrId */
	__put_u8(pdata, UnsignedInteger_16);
	pdata += sizeof(DataTypeDescriptor);
	pduForm->dataSize = sizeof(DataTypeDescriptor);

	__put_u16(pdata, htons(svcAttrId));
	pdata += sizeof(ServiceAttributeIdentifier);
	pduForm->dataSize += sizeof(ServiceAttributeIdentifier);
}

/*
 ** Generate the attribute sequence pdu form
 ** from slist_t elements. Return length of attr seq
 */
int generateDataSequencePduForm(char *dstBuffer, slist_t *seq, DataTypeDescriptor dataType)
{
	SDPData		*pDataSeq, *data;
	PDUForm		pduForm;
	int		seqLength = 0, i;
	slist_t		*dataSeq = NULL; 
	
	BTDEBUG("gen sequence\n");
	memset(&pduForm, 0, sizeof(PDUForm));
	pduForm.data = (char *)malloc(UUIDSequenceSize);
	if (!pduForm.data)
		return 0;
		
	pduForm.bufferSize = UUIDSequenceSize;

	seqLength = s_list_length(seq);
	BTDEBUG("Seq length : %d\n", seqLength);
	for (i = 0; i < seqLength; i++) {
		void * ptr = s_list_nth_data(seq, i);
		if ((dataType == UUID_16) || (dataType == UUID_32) || (dataType == UUID_128))
			ptr = &((UUID *)ptr)->value;
		data = createSDPData(dataType, ptr);
		s_list_append(&dataSeq, data);
	}

	seqLength = 0;
	pDataSeq = createSDPData(SequenceSize8, dataSeq);
	BTDEBUG("Data Seq : %p\n", pDataSeq);
	seqLength = generatePDUForm(&pduForm, pDataSeq);
	BTDEBUG("Copying : %d\n", pduForm.dataSize);
	memcpy(dstBuffer, pduForm.data, pduForm.dataSize);

	free(pDataSeq);
	free(pduForm.data);
	return seqLength;
}

int generateSearchSequencePduForm(char *dstBuffer, slist_t *seq)
{
	UUID	*pUUID = (UUID *)s_list_nth_data(seq, 0);
	return generateDataSequencePduForm(dstBuffer, seq, pUUID->uuidType);
}

int generateAttrIDSequencePduForm(char *dstBuffer, slist_t *seq, DataTypeDescriptor dataType)
{
	return generateDataSequencePduForm(dstBuffer, seq, dataType);
}



int generateDataSeqPDUForm(PDUForm *dstPduForm,	SDPData *pSDPData)
{
	SDPData		*localSDPData;
	slist_t		*dataSeq;
	int		dataSize = 0;

	for (dataSeq = pSDPData->sdpDataValue.dataSeq; dataSeq; dataSeq = s_list_next(dataSeq)) {
		localSDPData = dataSeq->data;
		dataSize += generatePDUForm(dstPduForm, localSDPData);
#ifdef CONFIG_AFFIX_DEBUG
		printPduForm(dstPduForm);
#endif
	}

	return dataSize;
}

int generatePDUForm(PDUForm *dstPduForm, SDPData *pSDPData)
{
	int		pduSize = 0;
	int 		dataSize = 0;
	unsigned char 	*pSrc = NULL;
	unsigned char 	sequenceType = 0;
	unsigned char 	sequenceAlternatesType = 0;
	DataTypeDescriptor	dtd;
	DataTypeDescriptor 	savedDTD;
	char 		*sequenceStartPtr = NULL;
	uint16_t 	uint16nbo;
	uint32_t 	uint32nbo;

	dtd = pSDPData->dtd;

	if (pSDPData == NULL)
		return 0;

	sequenceStartPtr = dstPduForm->data + dstPduForm->dataSize;
	pduSize = setDataTypeDescriptor(dstPduForm, dtd);
	switch (dtd) {
		case UnsignedInteger_8 :
			pSrc = &pSDPData->sdpDataValue.uint8;
			dataSize = sizeof(uint8_t);
			break;
		case UnsignedInteger_16 :
			uint16nbo = htons(pSDPData->sdpDataValue.uint16);
			pSrc = (unsigned char *)&uint16nbo;
			dataSize = sizeof(uint16_t);
			break;
		case UnsignedInteger_32 :
			uint32nbo = htonl(pSDPData->sdpDataValue.uint32);
			pSrc = (unsigned char *)&uint32nbo;
			dataSize = sizeof(uint32_t);
			break;
		case UnsignedInteger_64 :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.uint64;
			dataSize = sizeof(uint64_t);
			break;
		case UnsignedInteger_128 :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.uint128;
			dataSize = sizeof(Int128Bit);
			break;
		case Integer_8 :
		case Boolean :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.int8;
			dataSize = sizeof(int8_t);
			break;
		case Integer_16 :
			uint16nbo = htons(pSDPData->sdpDataValue.int16);
			pSrc = (unsigned char *)&uint16nbo;
			dataSize = sizeof(int16_t);
			break;
		case Integer_32 :
			uint32nbo = htonl(pSDPData->sdpDataValue.int32);
			pSrc = (unsigned char *)&uint32nbo;
			dataSize = sizeof(int32_t);
			break;
		case Integer_64 :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.int64;
			dataSize = sizeof(int64_t);
			break;
		case Integer_128 :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.int128;
			dataSize = sizeof(Int128Bit);
			break;

		case TextStringSize8 :
		case URLStringSize8 :
		case TextStringSize16 :
		case TextStringSize32 :
		case URLStringSize16 :
		case URLStringSize32 :
			pSrc = (unsigned char *)pSDPData->sdpDataValue.stringPtr;
			dataSize = strlen(pSDPData->sdpDataValue.stringPtr);
			setSequenceLength(sequenceStartPtr, dataSize);
			break;

		case SequenceSize8 :
		case SequenceSize16 :
		case SequenceSize32 :
			sequenceType = 1;
			dataSize = 0;
			dataSize = generateDataSeqPDUForm(dstPduForm, pSDPData);
			setSequenceLength(sequenceStartPtr, dataSize);
			break;

		case AlternativeSize8:
		case AlternativeSize16 :
		case AlternativeSize32 :
			sequenceAlternatesType = 1;
			savedDTD = pSDPData->dtd;
			pSDPData->dtd = SequenceSize8;
			dataSize = generatePDUForm(dstPduForm, pSDPData);
			pSDPData->dtd = savedDTD;
			setSequenceLength(sequenceStartPtr, dataSize);
			break;

		case UUID_16 :
			uint16nbo = htons(pSDPData->sdpDataValue.uuid.value.uuid16Bit);
			pSrc = (unsigned char *)&uint16nbo;
			dataSize = sizeof(UUID16Bit);
			break;

		case UUID_32 :
			uint32nbo = htonl(pSDPData->sdpDataValue.uuid.value.uuid32Bit);
			pSrc = (unsigned char *)&uint32nbo;
			dataSize = sizeof(UUID32Bit);
			break;

		case UUID_128 :
			pSrc = (unsigned char *)&pSDPData->sdpDataValue.uuid.value.uuid128Bit;
			dataSize = sizeof(UUID128Bit);
			break;

		default :
			break;
	}

	if ((sequenceType == 0) && (sequenceAlternatesType == 0)) {
		if ((pSrc != NULL) && (dstPduForm != NULL)) {
			memcpy((dstPduForm->data + dstPduForm->dataSize), pSrc, dataSize);
			dstPduForm->dataSize += dataSize;
		} else
			BTDEBUG("Gen PDU : Cant copy from NULL source or dest\n");
	}
	pduSize += dataSize;

	return pduSize;
}


void attrPduForm(void * value, void * userData)
{
	SDPData			*pSDPData;
	DataTypeDescriptor	dtd;
	int			pduFormSize = 0;

	if (value == NULL)
		return;

	pSDPData = (SDPData *)value;

	dtd = pSDPData->dtd;

	if (dtd < UUID_128)
		pduFormSize = BasicAttributePduFormSize;
	else
		pduFormSize = SequencePduFormSize;

	if (pSDPData->pduForm.data == NULL) {
		pSDPData->pduForm.data = (char *)malloc(pduFormSize);
		if (!pSDPData->pduForm.data)
			return;
		pSDPData->pduForm.bufferSize = pduFormSize;
	}
	pSDPData->pduForm.dataSize = 0;
	memset(pSDPData->pduForm.data, 0, pSDPData->pduForm.bufferSize);

#ifdef CONFIG_AFFIX_DEBUG
	printPduForm(&pSDPData->pduForm);
	BTDEBUG("Adding attr id : 0x%x", pSDPData->attrId);
#endif
	
	setAttributeId(&pSDPData->pduForm, pSDPData->attrId);
	generatePDUForm(&pSDPData->pduForm, pSDPData);

#ifdef CONFIG_AFFIX_DEBUG
	printPduForm(&pSDPData->pduForm);
#endif

	/*
	 ** Append to the overall PDU form
	 */
	if (userData != NULL)
		appendToPduForm((PDUForm *)userData, &pSDPData->pduForm);
}

/*
 *  Main function. Called to serialize Service Record to PDUForm
 */
void generateServiceRecordPduForm(ServiceRecord *svcRec)
{
	slist_t *attrList = NULL;

	attrList = svcRec->attributeList;
	if (attrList == NULL)
		return;

	if (svcRec->pduForm.data == NULL) {
		svcRec->pduForm.data = (char *)malloc(SDP_PDU_CHUNK_SIZE);
		svcRec->pduForm.bufferSize = SDP_PDU_CHUNK_SIZE;
	}
	svcRec->pduForm.dataSize = 0;
	memset(svcRec->pduForm.data, 0, svcRec->pduForm.bufferSize);
	s_list_foreach(attrList, attrPduForm, &svcRec->pduForm);
	//setSequenceLengthPDU(&svcRec->pduForm);
#ifdef CONFIG_AFFIX_DEBUG
	printPduForm(&svcRec->pduForm);
#endif
}


int addToAttributeList(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId,
		SDPData *pSDPData, int replace)
{
	SDPData *pData = NULL;

	BTDEBUG("Adding attr id : 0x%x to attr table", attrId);

	pSDPData->attrId = attrId;
	pData = getSDPData(svcRec, attrId);
	if (pData != NULL) {
		if (replace) {
			s_list_remove(&svcRec->attributeList, pData);
			freeSDPData(pData);
			pData = NULL;
		} else {
			BTDEBUG("Attribute with id : 0x%x already exists in table : 0x%x\n",
					attrId, (uint32_t)svcRec);
			return -1;

		}
	} 
	s_list_insert_sorted(&svcRec->attributeList, pSDPData, attrIdCompareFunc);
	return 0;
}



void printInteger(SDPData *pData)
{
	if (pData != NULL)
		BTDEBUG("Integer : 0x%x", pData->sdpDataValue.uint32);
}

void printString(SDPData *pData)
{
	if (pData != NULL)
		BTDEBUG("String : %s", pData->sdpDataValue.stringPtr);
}

void printDataSeq(slist_t *dataSeq)
{
	SDPData		*pSDPData;

	for (;dataSeq; dataSeq = s_list_next(dataSeq)) {
		pSDPData = dataSeq->data;
		printSDPData(pSDPData);
	}
}

void printSDPDataAsServiceRecord(ServiceRecord *svcRec)
{
	ServiceAttributeIdentifier attrId;
	SDPData *pSDPData;

	/*
	// print the service record handle 
	attrId = AttrServiceRecordHandle;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
	printf("Service Record Handle: 0x%x\n", pSDPData->sdpDataValue.uint32);
	}
	*/

	// print the service name
	attrId = AttrServiceName_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Name: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
	attrId = AttrServiceDescription_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Description: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
	attrId = AttrServiceProviderName_Primary;
	pSDPData = getSDPData(svcRec, attrId);   
	if (pSDPData != NULL) {
		printf("Service Provider: %s\n", pSDPData->sdpDataValue.stringPtr);
	}
}
void printSDPData(SDPData *pSDPData)
{
	DataTypeDescriptor dtd;

	dtd = pSDPData->dtd;
	switch (dtd) {
		case Boolean :
		case UnsignedInteger_8 :
		case UnsignedInteger_16 :
		case UnsignedInteger_32 :
		case UnsignedInteger_64 :
		case UnsignedInteger_128 :
		case Integer_8 :
		case Integer_16 :
		case Integer_32 :
		case Integer_64 :
		case Integer_128 :
			printInteger(pSDPData);
			break;

		case UUID_16 :
		case UUID_32 :
		case UUID_128 :
			BTDEBUG("UUID");
			printUUID(&pSDPData->sdpDataValue.uuid);
			break;

		case TextStringSize8 :
		case TextStringSize16 :
		case TextStringSize32 :
			BTDEBUG("Text : %s", pSDPData->sdpDataValue.stringPtr);
			break;
		case URLStringSize8 :
		case URLStringSize16 :
		case URLStringSize32 :
			BTDEBUG("URL : %s", pSDPData->sdpDataValue.stringPtr);
			break;

		case SequenceSize8 :
		case SequenceSize16 :
		case SequenceSize32 :
			printDataSeq(pSDPData->sdpDataValue.dataSeq);
			break;

		case AlternativeSize8 :
		case AlternativeSize16 :
		case AlternativeSize32 :
			BTDEBUG("Data Sequence Alternates");
			printDataSeq(pSDPData->sdpDataValue.dataSeq);
			break;
	}
}

void attrPrintFunc(void * value, void * userData)
{
	SDPData *pSDPData = NULL;
	ServiceAttributeIdentifier attrId;

	BTDEBUG("[%s-%d] : ", __FUNCTION__, __LINE__);
	pSDPData = (SDPData *)value;
	attrId = pSDPData->attrId;
	BTDEBUG("=====================================");
	BTDEBUG("ATTRIBUTE IDENTIFIER : 0x%x", attrId);
	BTDEBUG("ATTRIBUTE VALUE PTR : 0x%x", (uint32_t)value);
	if (pSDPData != NULL) {
		printSDPData(pSDPData);
	} else
		BTDEBUG("NULL value");
	BTDEBUG("=====================================");
}

void printServiceAttributes(slist_t *svcAttrList)
{
	s_list_foreach(svcAttrList, attrPrintFunc, NULL);
}


int attrIdCompareFunc(const void * key1, const void * key2)
{
	int cmp = 0;
	SDPData *pData1 = NULL;
	SDPData *pData2 = NULL;

	pData1 = (SDPData *)key1;
	pData2 = (SDPData *)key2;
	if ((pData1 != NULL) && (pData2 != NULL)) {
		cmp = pData1->attrId - pData2->attrId;
	}

	return cmp;
}

SDPData *getSDPData(ServiceRecord *svcRec, ServiceAttributeIdentifier attrId)
{
	SDPData		sdpTemplate;
	slist_t		*pStartElem = NULL;

	if (svcRec->attributeList == NULL)
		return NULL;

	sdpTemplate.attrId = attrId;
	pStartElem = s_list_find_custom(svcRec->attributeList,
			&sdpTemplate, attrIdCompareFunc); 
	if (pStartElem != NULL)
		return (SDPData *)pStartElem->data;

	return NULL;
}

